import os
import pwd
from typing import Any, Callable, Dict, Iterator, List

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem


@pytest.fixture
def mocked_users() -> Iterator[Dict[str, Any]]:
    yield {}


@pytest.fixture
def mock_users(
    fs: FakeFilesystem, mocked_users: Dict[str, Any], monkeypatch: Any
) -> Iterator[Callable[[List[str]], Dict[str, Any]]]:
    def _mock_users(users: List[str]) -> Dict[str, Any]:
        class pwent:
            def __init__(self, user: str) -> None:
                self.pw_name = user
                self.pw_passwd = "*"
                self.pw_uid = 42
                self.pw_gid = 42
                self.pw_gecos = user
                self.pw_dir = f"/home/{user}"
                self.pw_shell = "/bin/sh"

        for u in users:
            pwe = pwent(u)
            os.makedirs(pwe.pw_dir, exist_ok=True)
            mocked_users[u] = pwe

        def getpwnam(user: str) -> object:
            return mocked_users[user]

        def getpwall() -> List[object]:
            return list(mocked_users.values())

        monkeypatch.setattr(pwd, "getpwnam", getpwnam)
        monkeypatch.setattr(pwd, "getpwall", getpwall)
        return mocked_users

    yield _mock_users
