from gcs_migration.parsers import HPSSConfig


# Test load from default path
def test_load_config_from_default_path(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
):
    # This is not a valid test if these values are None
    assert hpss_config_file_default_contents["LoginName"] is not None
    assert hpss_config_file_default_contents["Authenticator"] is not None
    assert hpss_config_file_default_contents["AuthenticationMech"] is not None
    assert hpss_config_file_default_contents["UDAChecksumSupport"] is not None

    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_config = HPSSConfig()

    assert hpss_config.login_name == hpss_config_file_default_contents["LoginName"]
    assert (
        hpss_config.authenticator == hpss_config_file_default_contents["Authenticator"]
    )
    assert (
        hpss_config.authentication_mech
        == hpss_config_file_default_contents["AuthenticationMech"]
    )
    assert (
        hpss_config.uda_checksum_support
        == hpss_config_file_default_contents["UDAChecksumSupport"]
    )


# Test load form custom path
def test_load_config_from_custom_path(
    hpss_write_config_file,
    hpss_config_file_default_contents,
):
    # This is not a valid test if these values are None
    assert hpss_config_file_default_contents["LoginName"] is not None
    assert hpss_config_file_default_contents["Authenticator"] is not None
    assert hpss_config_file_default_contents["AuthenticationMech"] is not None
    assert hpss_config_file_default_contents["UDAChecksumSupport"] is not None

    hpss_write_config_file("/foo", hpss_config_file_default_contents)
    hpss_config = HPSSConfig("/foo")

    assert hpss_config.login_name == hpss_config_file_default_contents["LoginName"]
    assert (
        hpss_config.authenticator == hpss_config_file_default_contents["Authenticator"]
    )
    assert (
        hpss_config.authentication_mech
        == hpss_config_file_default_contents["AuthenticationMech"]
    )
    assert (
        hpss_config.uda_checksum_support
        == hpss_config_file_default_contents["UDAChecksumSupport"]
    )


# Test login_name default
def test_default_for_login_name(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
    hpss_config_default_login_name,
):
    hpss_config_file_default_contents.pop("LoginName", None)
    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_config = HPSSConfig()
    assert hpss_config.login_name == hpss_config_default_login_name


# Test uda default
def test_default_for_uda_checksum_support(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
    hpss_config_default_uda_checksum,
):
    hpss_config_file_default_contents.pop("UDAChecksumSupport", None)
    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_config = HPSSConfig()
    assert hpss_config.uda_checksum_support == hpss_config_default_uda_checksum


# Test hpss_config > hpss_env
def test_config_file_values_are_priority_over_env_file_values(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
    hpss_write_env_file,
    hpss_env_file_default_path,
    hpss_env_file_default_contents,
):
    # This is not a valid test if these values are None
    assert hpss_config_file_default_contents["Authenticator"] is not None
    assert hpss_config_file_default_contents["AuthenticationMech"] is not None
    assert (
        hpss_env_file_default_contents["HPSS_API_AUTHN_MECH"] is not None
        or hpss_env_file_default_contents["HPSS_PRIMARY_AUTHN_MECH"] is not None
    )
    assert hpss_env_file_default_contents["HPSS_PRIMARY_AUTHENTICATOR"] is not None

    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_write_env_file(hpss_env_file_default_path, hpss_env_file_default_contents)
    hpss_config = HPSSConfig()

    assert (
        hpss_config.authenticator == hpss_config_file_default_contents["Authenticator"]
    )
    assert (
        hpss_config.authentication_mech
        == hpss_config_file_default_contents["AuthenticationMech"]
    )


# Test hpss_env works as backup
def test_env_file_used_as_backup(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
    hpss_write_env_file,
    hpss_env_file_default_path,
    hpss_env_file_default_contents,
):
    # This is not a valid test if these values are None
    assert hpss_env_file_default_contents["HPSS_API_AUTHN_MECH"] is not None
    assert hpss_env_file_default_contents["HPSS_PRIMARY_AUTHENTICATOR"] is not None

    hpss_config_file_default_contents.pop("AuthenticationMech", None)
    hpss_config_file_default_contents.pop("Authenticator", None)
    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_write_env_file(hpss_env_file_default_path, hpss_env_file_default_contents)
    hpss_config = HPSSConfig()

    assert (
        hpss_config.authenticator
        == hpss_env_file_default_contents["HPSS_PRIMARY_AUTHENTICATOR"]
    )
    assert (
        hpss_config.authentication_mech
        == hpss_env_file_default_contents["HPSS_API_AUTHN_MECH"]
    )


# Test HPSS_API_AUTHN_MECH > HPSS_PRIMARY_AUTHN_MECH
def test_env_auth_mech_falls_back_to_authn_mech(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
    hpss_write_env_file,
    hpss_env_file_default_path,
    hpss_env_file_default_contents,
):
    # This is not a valid test if these values are None
    assert hpss_env_file_default_contents["HPSS_PRIMARY_AUTHENTICATOR"] is not None
    assert hpss_env_file_default_contents["HPSS_PRIMARY_AUTHN_MECH"] is not None

    hpss_config_file_default_contents.pop("AuthenticationMech", None)
    hpss_config_file_default_contents.pop("Authenticator", None)
    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    hpss_env_file_default_contents.pop("HPSS_API_AUTHN_MECH", None)
    hpss_write_env_file(hpss_env_file_default_path, hpss_env_file_default_contents)
    hpss_config = HPSSConfig()

    assert (
        hpss_config.authenticator
        == hpss_env_file_default_contents["HPSS_PRIMARY_AUTHENTICATOR"]
    )
    assert (
        hpss_config.authentication_mech
        == hpss_env_file_default_contents["HPSS_PRIMARY_AUTHN_MECH"]
    )
